package net.wanji.web.service.impl;

import cn.hutool.core.util.ObjectUtil;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.gts.GeometryUtil;
import net.wanji.common.utils.tool.CrossUtil;
import net.wanji.databus.dao.mapper.BaseAreaInfoMapper;
import net.wanji.databus.dao.mapper.BaseCrossInfoMapper;
import net.wanji.databus.po.BaseAreaInfoPO;
import net.wanji.databus.po.BaseCrossInfoPO;
import net.wanji.web.bo.AddOrUpdateAreaBO;
import net.wanji.web.bo.AreaIdBO;
import net.wanji.web.bo.PolygonBO;
import net.wanji.web.bo.SpecialServiceRouteBO;
import net.wanji.web.mapper.TBaseAreaCrossMapper;
import net.wanji.web.service.CrossManageService;
import net.wanji.web.vo.CrossIdAndNameVO;
import net.wanji.web.vo.SelectCrossesByStartEndVO;
import net.wanji.web.vo.specialService.RouteElementVO;
import org.locationtech.jts.geom.Geometry;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Kent HAN
 * @date 2023/6/25 8:56
 */
@Service
@Slf4j
public class CrossManageServiceImpl implements CrossManageService {
    private final BaseCrossInfoMapper crossInfoMapper;
    private final BaseAreaInfoMapper baseAreaInfoMapper;
    private final SpecialServiceServiceImpl specialServiceService;
    private final TBaseAreaCrossMapper areaCrossMapper;

    public CrossManageServiceImpl(BaseCrossInfoMapper crossInfoMapper, BaseAreaInfoMapper baseAreaInfoMapper,
                                  SpecialServiceServiceImpl specialServiceService, TBaseAreaCrossMapper areaCrossMapper) {
        this.crossInfoMapper = crossInfoMapper;
        this.baseAreaInfoMapper = baseAreaInfoMapper;
        this.specialServiceService = specialServiceService;
        this.areaCrossMapper = areaCrossMapper;
    }

    @Override
    public List<CrossIdAndNameVO> selectCrossesByPolygon(PolygonBO polygonBO) {
        List<CrossIdAndNameVO> res = new ArrayList<>();
        String wkt = polygonBO.getWkt();
        List<BaseCrossInfoPO> crossList = crossInfoMapper.selectAll();

        for (BaseCrossInfoPO cross : crossList) {
            String location = cross.getLocation();
            double[] lonLat = CrossUtil.getLonLat(location);
            String crossLonLat = lonLat[0] + "," + lonLat[1];
            Geometry wktGeo = GeometryUtil.str2Geometry(wkt);
            Geometry crossGeo = GeometryUtil.str2Geometry(crossLonLat);
            // 判断点和多边形是否相交
            boolean isIntersect = wktGeo.intersects(crossGeo);
            if (isIntersect) {
                CrossIdAndNameVO crossIdAndNameVO = new CrossIdAndNameVO();
                crossIdAndNameVO.setCrossId(cross.getId());
                crossIdAndNameVO.setCrossName(cross.getName());
                res.add(crossIdAndNameVO);
            }
        }

        return res;
    }

    @Override
    @Transactional
    public void addOrUpdateArea(AddOrUpdateAreaBO addOrUpdateAreaBO) {
        Integer type = addOrUpdateAreaBO.getType();
        Integer areaId = addOrUpdateAreaBO.getAreaId();
        Integer parentCode = addOrUpdateAreaBO.getParentCode();
        String areaName = addOrUpdateAreaBO.getAreaName();
        String roadName = addOrUpdateAreaBO.getRoadName();
        String remark = addOrUpdateAreaBO.getRemark();
        List<String> crossIdList = addOrUpdateAreaBO.getCrossIdList();
        String wkt = addOrUpdateAreaBO.getWkt();

        if (ObjectUtil.isNotEmpty(areaId) && 0 != areaId) {
            // 修改
            // 修改区域基础信息
            baseAreaInfoMapper.updateAreaNameAndRemark(areaName, remark, areaId, roadName, parentCode);
            // 修改区域关联路口
            areaCrossMapper.deleteByAreaId(areaId);
            for (String crossId : crossIdList) {
                areaCrossMapper.insertOne(crossId, areaId);
            }
        } else {
            // 新增
            if (2 == type) { // 交警辖区
                BaseAreaInfoPO baseAreaInfoPO = new BaseAreaInfoPO();
                if (parentCode != null) {
                    baseAreaInfoPO.setParentCode(parentCode);
                } else {
                    baseAreaInfoPO.setParentCode(0);
                }
                baseAreaInfoPO.setRoadName("");
                handleAreaInfoPO(type, areaName, remark, crossIdList, wkt, baseAreaInfoPO);
            } else if (6 == type) { // 道路
                BaseAreaInfoPO baseAreaInfoPO = new BaseAreaInfoPO();
                baseAreaInfoPO.setRoadName(roadName);
                baseAreaInfoPO.setParentCode(0);
                handleAreaInfoPO(type, areaName, remark, crossIdList, wkt, baseAreaInfoPO);
            } else if (5 == type) { // 子区
                BaseAreaInfoPO baseAreaInfoPO = new BaseAreaInfoPO();
                baseAreaInfoPO.setRoadName("");
                baseAreaInfoPO.setParentCode(0);
                handleAreaInfoPO(type, areaName, remark, crossIdList, wkt, baseAreaInfoPO);
            }
        }
    }

    private void handleAreaInfoPO(Integer type, String areaName, String remark, List<String> crossIdList, String wkt,
                                  BaseAreaInfoPO baseAreaInfoPO) {
        baseAreaInfoPO.setCode(0);
        baseAreaInfoPO.setLocation("");
        baseAreaInfoPO.setName(areaName);
        baseAreaInfoPO.setType(type);
        baseAreaInfoPO.setPolylines(wkt);
        baseAreaInfoPO.setRemark(remark);
        baseAreaInfoMapper.insertOne(baseAreaInfoPO);
        Integer areaIdGenerated = baseAreaInfoPO.getId();
        for (String crossId : crossIdList) {
            areaCrossMapper.insertOne(crossId, areaIdGenerated);
        }
    }

    @Override
    public SelectCrossesByStartEndVO selectCrossesByStartEnd(SpecialServiceRouteBO specialServiceRouteBO) {
        SelectCrossesByStartEndVO selectCrossesByStartEndVO = new SelectCrossesByStartEndVO();
        StringBuilder wkt = new StringBuilder();

        List<List<RouteElementVO>> twoRoutes = specialServiceService.specialServiceRoute(specialServiceRouteBO);;
        List<RouteElementVO> route = twoRoutes.get(0); // 选最优路线

        List<CrossIdAndNameVO> crossIdAndNameVOList = new ArrayList<>();
        for (int i = 0; i < route.size(); i++) {
            RouteElementVO routeElementVO = route.get(i);
            String wktPart = routeElementVO.getWkt();
            wkt.append(wktPart);
            CrossIdAndNameVO crossIdAndNameVO = new CrossIdAndNameVO();
            crossIdAndNameVO.setCrossId(routeElementVO.getStartId());
            crossIdAndNameVO.setCrossName(routeElementVO.getStartName());
            crossIdAndNameVOList.add(crossIdAndNameVO);
            if (i == route.size() - 1) { // 最后一段
                CrossIdAndNameVO crossIdAndNameVOEnd = new CrossIdAndNameVO();
                crossIdAndNameVOEnd.setCrossId(routeElementVO.getEndId());
                crossIdAndNameVOEnd.setCrossName(routeElementVO.getEndName());
                crossIdAndNameVOList.add(crossIdAndNameVOEnd);
            }
        }

        selectCrossesByStartEndVO.setWkt(wkt.toString());
        selectCrossesByStartEndVO.setCrossIdAndNameVOList(crossIdAndNameVOList);
        return selectCrossesByStartEndVO;
    }

    @Override
    @Transactional
    public void deleteArea(AreaIdBO areaIdBO) {
        Integer areaId = areaIdBO.getAreaId();
        // 删除区域路口关系
        List<Integer> areaIds = baseAreaInfoMapper.selectIdsByParentCode(areaId);
        areaIds.add(areaId);
        areaCrossMapper.deleteByAreaIds(areaIds);
        // 删除区域基础信息
        baseAreaInfoMapper.deleteById(areaId);
    }
}
